# pylint: skip-file
# mypy: ignore-errors

import subprocess


def main() -> int:
    subprocess.Popen(
        r'"C:\Program Files\Git\git-bash.exe"',
        shell=True,
        start_new_session=True,
        creationflags=subprocess.CREATE_NEW_CONSOLE,  # pylint: disable
    )


if __name__ == "__main__":
    main()



import os
import shutil
import sys

from zcmds_win32._exec import os_exec


def get_sublime() -> str | None:
    """Attempts to find the Sublime Text executable."""
    path = shutil.which("subl") or shutil.which("sublime_text")
    if path:
        return path
    for i in ["", "2", "3", "4"]:
        path = f"C:\\Program Files\\Sublime Text{i}\\sublime_text.exe"
        if os.path.exists(path):
            return path
    return None


def get_textpad() -> str | None:
    path = shutil.which("textpad")
    if path:
        return path
    for i in ["", "2", "3", "4", "5", "6", "7", "8"]:
        path = f"C:\\Program Files\\TextPad {i}\\TextPad.exe"
        if os.path.exists(path):
            return path
    return None


TEXT_EDITOR = get_sublime() or get_textpad()

SOURCE_EXTENSIONS = [
    ".c",
    ".cpp",
    ".cxx",
    ".cc",
    ".c++",
    ".h",
    ".hpp",
    ".hxx",
    ".hh",
    ".h++",
    ".py",
    ".kt",
    ".java",
    ".js",
    ".ts",
    ".html",
    ".css",
    ".scss",
    ".sass",
    ".less",
    ".json",
    ".xml",
    ".yml",
    ".yaml",
    ".md",
    ".txt",
    ".bat",
    ".cmd",
    ".sh",
    ".ps1",
]


def handle_file(file: str) -> tuple[bool, int]:
    """Attempts to handle the file with a specific program."""
    ext = os.path.splitext(file)[1]
    if ext in SOURCE_EXTENSIONS:
        if TEXT_EDITOR:
            # empty quotes is for title.
            cmd = f'start "" "{TEXT_EDITOR}" "{file}"'
            rtn = os.system(cmd)
            return (True, rtn)
    return (False, 0)


def main() -> int:
    cmd = "explorer"
    if len(sys.argv) == 1:
        cmd += " ."
        return os.system(cmd)
    for i, _ in enumerate(sys.argv):
        if i < 1:
            continue
        arg = sys.argv[i].replace("/", "\\")
        sys.argv[i] = arg
        if os.path.isfile(arg):
            handled, ret = handle_file(arg)
            if handled:
                return ret
    return os_exec(cmd)



"""
Resolves the unix tool, either by finding it in PATH or by downloading it.
"""

import os
import shutil
from tempfile import TemporaryDirectory
from typing import Optional

from download import download  # type: ignore

from zcmds_win32._exec import os_exec

GIT_BIN = r"C:\Program Files\Git\usr\bin"
GIT_BIN_TOOL_URL = (
    "https://github.com/zackees/zcmds_win32/raw/main/assets/git-bash-bin.zip"
)

HERE = os.path.abspath(os.path.dirname(__file__))
DOWNLOADED_GIT_BIN = os.path.join(HERE, "git-bash-bin")


def install() -> None:
    """Installs the Unix tools."""
    with TemporaryDirectory() as tmpdir:
        download(GIT_BIN_TOOL_URL, tmpdir, replace=True, kind="zip")
        dst = os.path.join(HERE, "git-bash-bin")
        os.makedirs(dst, exist_ok=True)
        files = os.listdir(os.path.join(tmpdir, "git-bash-bin"))
        # Sort dll's so that they are first
        files.sort(key=lambda x: not x.endswith(".dll"))
        files = [os.path.join(tmpdir, "git-bash-bin", f) for f in files]
        for src in files:
            filename = os.path.basename(src)
            try:
                shutil.move(src, os.path.join(dst, filename))
            except shutil.Error:
                pass


def get_or_fetch_unix_tool_path(name: str) -> Optional[str]:
    """Attempts to find the given Unix tool."""
    path = shutil.which(name)
    if path and os.path.basename(os.path.dirname(path)).lower() != "scripts":
        return path
    # add .exe to the name if it's not there
    if not name.lower().endswith(".exe"):
        name += ".exe"
    if os.path.exists(os.path.join(GIT_BIN, name)):
        return os.path.join(GIT_BIN, name)
    if not os.path.exists(DOWNLOADED_GIT_BIN):
        install()
    return os.path.join(HERE, "git-bash-bin", name)


def unix_tool_exec(
    cmdname: str, inherit_params: bool = True, cwd: Optional[str] = None
) -> int:
    """Executes the given Unix tool."""
    cmd = get_or_fetch_unix_tool_path(cmdname)
    if not cmd:
        raise FileNotFoundError(f"Could not find {cmdname} in PATH or in {GIT_BIN}")
    return os_exec(cmd, inherit_params, cwd)
